@@ -86,6 +86,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 
 		break;
 	}
+#ifdef TSS_BUILD_TICK
 	/* TPM BLOB: TPM_CURRENT_TICKS, UINT32, BLOB, optional AUTH */
 	case TPM_ORD_TickStampBlob:
 	{
@@ -130,6 +131,8 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 
 		break;
 	}
+#endif
+#ifdef TSS_BUILD_QUOTE
 	/* TPM BLOB: TPM_PCR_COMPOSITE, UINT32, BLOB, 1 optional AUTH
 	 * return UINT32*, BYTE**, UINT32*, BYTE**, 1 optional AUTH */
 	case TPM_ORD_Quote:
@@ -174,6 +177,8 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 
 		break;
 	}
+#endif
+#ifdef TSS_BUILD_TSS12
 	/* TPM BLOB: TPM_PCR_INFO_SHORT, (UINT32, BLOB,) UINT32, BLOB, 1 optional AUTH */
 	case TPM_ORD_Quote2:
 	{
@@ -247,6 +252,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 		UnloadBlob(&offset1, *len3, b, *blob3);
 		break;
 	}
+#endif
 	/* TPM BLOB: TPM_CERTIFY_INFO, UINT32, BLOB, 2 optional AUTHs
 	 * return UINT32*, BYTE**, UINT32*, BYTE**, 2 optional AUTHs */
 	case TPM_ORD_CertifyKey:
@@ -299,6 +305,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 
 		break;
 	}
+#ifdef TSS_BUILD_AUDIT
 	/* TPM_BLOB: TPM_COUNTER_VALUE, DIGEST, DIGEST, UINT32, BLOB, optional AUTH
 	 * return: UINT32*, BYTE**, DIGEST*, DIGEST*, UINT32*, BYTE**, optional AUTH */
 	case TPM_ORD_GetAuditDigestSigned:
@@ -389,6 +396,8 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 
 		break;
 	}
+#endif
+#ifdef TSS_BUILD_COUNTER
 	/* optional UINT32, TPM_COUNTER_VALUE, optional AUTH */
 	case TPM_ORD_ReadCounter:
 	case TPM_ORD_CreateCounter:
@@ -416,6 +425,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 
 		break;
 	}
+#endif
 	/* TPM BLOB: UINT32, BLOB, UINT32, BLOB, optional AUTH, optional AUTH */
 	case TPM_ORD_CreateMaintenanceArchive:
 	case TPM_ORD_CreateMigrationBlob:
@@ -458,6 +468,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 
 		UnloadBlob_UINT32(&offset1, len2, b);
 		if ((*blob2 = malloc(*len2)) == NULL) {
+			free(*blob1);
 			LogError("malloc of %u bytes failed", *len2);
 			return TCSERR(TSS_E_OUTOFMEMORY);
 		}
@@ -539,6 +550,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 		UnloadBlob_UINT32(&offset2, len2, b);
 
 		if ((*blob2 = malloc(*len2)) == NULL) {
+			free(*blob1);
 			LogError("malloc of %u bytes failed", *len2);
 			return TCSERR(TSS_E_OUTOFMEMORY);
 		}
@@ -587,7 +599,6 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 	case TPM_ORD_DAA_Sign:
 	case TPM_ORD_ChangeAuth:
 	case TPM_ORD_GetCapability:
-	case TPM_ORD_UnBind:
 	case TPM_ORD_LoadMaintenanceArchive:
 	case TPM_ORD_ConvertMigrationBlob:
 	case TPM_ORD_NV_ReadValue:
@@ -631,6 +642,35 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 		UnloadBlob(&offset1, *data_len, b, *data);
 		break;
 	}
+	/* TPM BLOB: 1 UINT32, 1 BLOB, 1 optional AUTH
+	* return: UINT32 *, BYTE**, 1 optional AUTH*/
+	case TPM_ORD_UnBind:
+	{
+		UINT32 *data_len = va_arg(ap, UINT32 *);
+		BYTE **data = va_arg(ap, BYTE **);
+		TPM_AUTH *auth1 = va_arg(ap, TPM_AUTH *);
+		va_end(ap);
+
+		if (!data || !data_len) {
+			LogError("Internal error for ordinal 0x%x", ordinal);
+			return TCSERR(TSS_E_INTERNAL_ERROR);
+		}
+
+		if (auth1) {
+			offset1 = len - TSS_TPM_RSP_BLOB_AUTH_LEN;
+			UnloadBlob_Auth(&offset1, b, auth1);
+		} 
+
+		offset1 = TSS_TPM_TXBLOB_HDR_LEN;
+		UnloadBlob_UINT32(&offset1, data_len, b);
+		if ((*data = malloc(*data_len)) == NULL) {
+			LogError("malloc of %u bytes failed", *data_len);
+			return TCSERR(TSS_E_OUTOFMEMORY);
+		}
+
+		UnloadBlob(&offset1, *data_len, b, *data);
+		break;
+	}
 	/* TPM BLOB: 1 BLOB, 1 optional AUTH
 	 * return: UINT32 *, BYTE**, 1 optional AUTH*/
 	case TPM_ORD_GetTicks:
@@ -705,6 +745,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 		*data_len = offset2;
 		break;
 	}
+#ifdef TSS_BUILD_TSS12
 	/* TPM BLOB: BLOB, DIGEST, DIGEST
 	 * return: UINT32 *, BYTE**, DIGEST, DIGEST */
 	case TPM_ORD_CreateRevocableEK:
@@ -737,6 +778,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 		*data_len = offset2;
 		break;
 	}
+#endif
 	/* 1 UINT32, 1 optional AUTH */
 	case TPM_ORD_LoadKey:
 	case TPM_ORD_LoadKey2:
@@ -805,6 +847,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 		UnloadBlob(&offset1, TPM_NONCE_SIZE, b, nonce2);
 		break;
 	}
+#ifdef TSS_BUILD_CMK
 	/* 1 20 byte value, 1 optional AUTH */
 	case TPM_ORD_CMK_ApproveMA:
 	case TPM_ORD_CMK_CreateTicket:
@@ -826,6 +869,7 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 		}
 		break;
 	}
+#endif
 	/* 1 optional AUTH */
 	case TPM_ORD_DisablePubekRead:
 	case TPM_ORD_DirWriteAuth:
@@ -871,7 +915,8 @@ tpm_rsp_parse(TPM_COMMAND_CODE ordinal, BYTE *b, UINT32 len, ...)
 TSS_RESULT
 tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 {
-	TSS_RESULT result = TCSERR(TSS_E_INTERNAL_ERROR);
+	TSS_RESULT result = TSS_SUCCESS;
+	UINT64 blob_size;
 	va_list ap;
 
 	DBG_ASSERT(ordinal);
@@ -881,6 +926,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 	va_start(ap, out_blob);
 
 	switch (ordinal) {
+#ifdef TSS_BUILD_DELEGATION
 	/* 1 UINT16, 1 UINT32, 1 20 bytes value, 1 UINT32, 1 BLOB */
 	case TPM_ORD_DSAP:
 	{
@@ -892,6 +938,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !in_blob1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -904,7 +951,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob(outOffset, in_len1, out_blob, in_blob1);
 		LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 BOOL, 1 UINT32, 1 BLOB, 1 20 byte value, 1 AUTH */
@@ -918,6 +964,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!in_len1 || !in_blob1 || !digest1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -932,7 +979,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		} else
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 2 UINT32's, 1 BLOB, 1 20 byte value, 1 AUTH */
@@ -946,6 +992,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keyslot1 || !in_len1 || !in_blob1 || !digest1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -960,9 +1007,10 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		} else
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
+#ifdef TSS_BUILD_TRANSPORT
 	/* 3 UINT32's, 1 BLOB, 2 AUTHs */
 	case TPM_ORD_ExecuteTransport:
 	{
@@ -998,9 +1046,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ord1, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	/* 1 UINT32, 1 UINT16, 1 BLOB, 1 UINT32, 1 BLOB, 1 options AUTH, 1 AUTH */
 	case TPM_ORD_CreateMigrationBlob:
 	{
@@ -1015,6 +1063,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!in_blob1 || !in_blob2 || !auth2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1034,7 +1083,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT32, 1 UINT16, 1 20 byte value, 1 UINT16, 1 UINT32, 1 BLOB, 2 AUTHs */
@@ -1051,6 +1099,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !in_blob1 || !auth1 || !auth2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1066,7 +1115,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth2);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH2_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 2 DIGEST/ENCAUTH's, 1 UINT32, 1 BLOB, 1 optional AUTH, 1 AUTH */
@@ -1085,6 +1133,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!dig1 || !dig2 || !blob1 || !auth2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1102,9 +1151,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#if (TSS_BUILD_NV || TSS_BUILD_DELEGATION)
 	/* 3 UINT32's, 1 BLOB, 1 optional AUTH */
 	case TPM_ORD_NV_WriteValue:
 	case TPM_ORD_NV_WriteValueAuth:
@@ -1118,6 +1167,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!in_blob1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1134,9 +1184,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	/* 3 UINT32's, 1 optional AUTH */
 	case TPM_ORD_NV_ReadValue:
 	case TPM_ORD_NV_ReadValueAuth:
@@ -1159,7 +1209,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 20 byte value, 1 UINT32, 1 BLOB */
@@ -1171,6 +1220,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !in_blob1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1180,9 +1230,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob(outOffset, in_len1, out_blob, in_blob1);
 		LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#ifdef TSS_BUILD_TSS12
 	/* 1 20 byte value, 1 UINT32, 1 BLOB, 1 BOOL, 1 20 byte value */
 	case TPM_ORD_CreateRevocableEK:
 	{
@@ -1194,6 +1244,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !in_blob1 || !digest2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1205,7 +1256,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob(outOffset, TPM_SHA1_160_HASH_LEN, out_blob, digest2);
 		LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 20 byte value */
@@ -1215,6 +1265,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1223,9 +1274,10 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob(outOffset, TPM_SHA1_160_HASH_LEN, out_blob, digest1);
 		LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
+#ifdef TSS_BUILD_COUNTER
 	/* 1 20 byte value, 1 UINT32, 1 BLOB, 1 AUTH */
 	case TPM_ORD_CreateCounter:
 	{
@@ -1236,6 +1288,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !in_blob1 || !auth1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1246,9 +1299,10 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth1);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
+#ifdef TSS_BUILD_DAA
 	/* 1 UINT32, 1 BYTE, 1 UINT32, 1 BLOB, 1 UINT32, 1 BLOB, 1 AUTH */
 	case TPM_ORD_DAA_Sign:
 	case TPM_ORD_DAA_Join:
@@ -1263,6 +1317,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keySlot1 || !in_blob1 || !auth1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1277,9 +1332,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth1);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	/* 2 UINT32's, 1 BLOB, 1 UINT32, 1 BLOB, 1 optional AUTH */
 	case TPM_ORD_ConvertMigrationBlob:
 	case TPM_ORD_SetCapability:
@@ -1293,6 +1348,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keySlot1 || !in_blob1 || !in_blob2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1310,7 +1366,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 2 UINT32's, 1 20 byte value, 2 optional AUTHs */
@@ -1324,6 +1379,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keySlot1 || !keySlot2 || !digest1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1346,7 +1402,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 2 UINT32's, 1 BLOB, 1 optional AUTH */
@@ -1362,6 +1417,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (in_len1 && !in_blob1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1378,7 +1434,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT32, 1 20 byte value, 1 UINT32, 1 optional BLOB, 1 UINT32, 1 BLOB, 1 AUTH */
@@ -1394,7 +1449,15 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		TPM_AUTH *auth1 = va_arg(ap, TPM_AUTH *);
 		va_end(ap);
 
+		blob_size = in_len1 + in_len2 + TPM_DIGEST_SIZE + sizeof(TPM_AUTH);
+		if (blob_size > TSS_TPM_TXBLOB_SIZE) {
+			result = TCSERR(TSS_E_BAD_PARAMETER);
+			LogError("Oversized input when building ordinal 0x%x", ordinal);
+			break;
+		}
+				
 		if (!keySlot1 || !in_blob2 || !auth1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1409,7 +1472,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth1);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 2 UINT32's, 1 BLOB, 1 optional AUTH, 1 AUTH */
@@ -1423,6 +1485,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keySlot1 || !in_blob1 || !auth2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1440,7 +1503,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT32, 1 20-byte blob, 1 BLOB, 1 optional AUTH */
@@ -1454,6 +1516,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keySlot1 || !digest1 || !in_blob1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1469,9 +1532,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		} else
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#ifdef TSS_BUILD_TSS12
 	/* 1 UINT32, 1 20-byte blob, 1 BLOB, 1 BOOL, 1 optional AUTH */
 	case TPM_ORD_Quote2:
 	{
@@ -1485,6 +1548,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keySlot1 || !digest1 || !in_blob1 || !addVersion) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1503,9 +1567,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		} else
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	/* 1 UINT32, 2 20-byte blobs, 1 BLOB, 1 optional AUTH */
 	case TPM_ORD_CreateWrapKey:
 	{
@@ -1518,6 +1582,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keySlot1 || !digest1 || !digest2 || !in_blob1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1533,7 +1598,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		} else
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 2 BLOBs, 1 optional AUTH */
@@ -1548,6 +1612,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!in_blob1 || !in_blob2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1562,9 +1627,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#ifdef TSS_BUILD_TICK
 	/* 1 UINT32, 2 20-byte blobs, 1 optional AUTH */
 	case TPM_ORD_TickStampBlob:
 	{
@@ -1575,6 +1640,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!keySlot1 || !digest1 || !digest2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1590,9 +1656,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		} else
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	/* 1 BLOB */
 	case TPM_ORD_ReadManuMaintPub:
 	case TPM_ORD_ReadPubek:
@@ -1604,6 +1670,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!in_blob1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1612,7 +1679,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob(outOffset, in_len1, out_blob, in_blob1);
 		LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT32, 1 BLOB, 2 optional AUTHs */
@@ -1636,6 +1702,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (in_len1 && !in_blob1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1657,7 +1724,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT16, 1 BLOB, 1 AUTH */
@@ -1670,6 +1736,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!in_blob1 || !auth1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1680,7 +1747,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth1);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT16, 1 UINT32, 1 BLOB, 1 UINT32, 2 BLOBs, 1 AUTH */
@@ -1697,6 +1763,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!in_blob1 || !in_blob2 || !in_blob3 || !auth1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1711,9 +1778,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth1);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#ifdef TSS_BUILD_AUDIT
 	/* 1 UINT32, 1 BOOL, 1 20 byte value, 1 optional AUTH */
 	case TPM_ORD_GetAuditDigestSigned:
 	{
@@ -1724,6 +1791,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1740,9 +1808,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	/* 1 UINT16, 1 UINT32, 1 20 byte value */
 	case TPM_ORD_OSAP:
 	{
@@ -1752,6 +1820,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1762,7 +1831,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob(outOffset, TPM_SHA1_160_HASH_LEN, out_blob, digest1);
 		LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT16, 1 20 byte value, 1 UINT16, 1 AUTH */
@@ -1775,6 +1843,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !auth1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1786,9 +1855,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth1);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#ifdef TSS_BUILD_AUDIT
 	/* 1 UINT32, 1 BOOL, 1 AUTH */
 	case TPM_ORD_SetOrdinalAuditStatus:
 	{
@@ -1798,6 +1867,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!auth1) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1808,9 +1878,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth1);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	/* 1 BOOL, 1 optional AUTH */
 	case TPM_ORD_OwnerSetDisable:
 	case TPM_ORD_PhysicalSetDeactivated:
@@ -1830,7 +1900,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 optional AUTH */
@@ -1862,7 +1931,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT32, 1 optional AUTH */
@@ -1892,9 +1960,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#ifdef TSS_BUILD_CMK
 	/* 1 20 byte value, 1 optional AUTH */
 	case TPM_ORD_CMK_ApproveMA:
 	{
@@ -1911,9 +1979,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	/* 1 UINT16 only */
 	case TSC_ORD_PhysicalPresence:
 	{
@@ -1924,9 +1992,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_UINT16(outOffset, i, out_blob);
 		LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#ifdef TSS_BUILD_CMK
 	/* 1 UINT32, 1 20 byte value, 1 BLOB, 2 20 byte values, 1 optional AUTH */
 	case TPM_ORD_CMK_CreateKey:
 	{
@@ -1940,6 +2008,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !in_blob1 || !digest2 || !digest3) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1957,7 +2026,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 BLOB, 1 20 byte value, 1 UINT32, 1 BLOB, 1 optional AUTH */
@@ -1972,6 +2040,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !in_blob1 || !in_blob2) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -1988,7 +2057,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT32, 1 UINT16, 1 BLOB, 1 20 byte value, 4 x (1 UINT32, 1 BLOB), 1 optional AUTH */
@@ -2011,6 +2079,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!digest1 || !in_blob1 || !in_blob2 || !in_blob3 || !in_blob4 || !in_blob5) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -2035,10 +2104,10 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
-	/* 1 UINT32, 1 60 byte value, 1 20 byte value, 1 BLOB, 2 x (1 UINT32, 1 BLOB), 1 optional AUTH */
+	/* 1 UINT32, 1 60 byte value, 1 20 byte value, 1 BLOB, 2 x (1 UINT32, 1 BLOB),
+	 * 1 optional AUTH */
 	case TPM_ORD_CMK_ConvertMigration:
 	{
 		UINT32 key1 = va_arg(ap, UINT32);
@@ -2054,6 +2123,7 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		va_end(ap);
 
 		if (!cmkauth1 || !digest1 || !in_blob1 || !in_blob2 || !in_blob3) {
+			result = TCSERR(TSS_E_INTERNAL_ERROR);
 			LogError("Internal error for ordinal 0x%x", ordinal);
 			break;
 		}
@@ -2074,9 +2144,10 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 			LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 		}
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
+#ifdef TSS_BUILD_TSS12
 	case TPM_ORD_FlushSpecific:
 	{
 		UINT32 val1 = va_arg(ap, UINT32);
@@ -2088,7 +2159,6 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_UINT32(outOffset, val2, out_blob);
 		LoadBlob_Header(TPM_TAG_RQU_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
 	/* 1 UINT32, 1 BLOB, 1 UINT32, 1 BOOL, 1 AUTH */
@@ -2110,9 +2180,9 @@ tpm_rqu_build(TPM_COMMAND_CODE ordinal, UINT64 *outOffset, BYTE *out_blob, ...)
 		LoadBlob_Auth(outOffset, out_blob, auth1);
 		LoadBlob_Header(TPM_TAG_RQU_AUTH1_COMMAND, *outOffset, ordinal, out_blob);
 
-		result = TSS_SUCCESS;
 		break;
 	}
+#endif
 	default:
 		LogError("Unknown ordinal: 0x%x", ordinal);
 		break;
