@@ -1512,7 +1512,8 @@ char_width(char *codeset)
 BYTE *
 Trspi_Native_To_UNICODE(BYTE *string, unsigned *size)
 {
-	char *ptr, *ret, *outbuf, tmpbuf[MAX_BUF_SIZE] = { 0, };
+	char *ret, *outbuf, tmpbuf[MAX_BUF_SIZE] = { 0, };
+	BSD_CONST char *ptr;
 	unsigned len = 0, tmplen;
 	iconv_t cd = 0;
 	size_t rc, outbytesleft, inbytesleft;
@@ -1538,7 +1539,7 @@ Trspi_Native_To_UNICODE(BYTE *string, unsigned *size)
 		ptr = (char *)string;
 		errno = 0;
 
-		rc = iconv(cd, &ptr, &inbytesleft, &outbuf, &outbytesleft);
+		rc = iconv(cd, (BSD_CONST char **)&ptr, &inbytesleft, &outbuf, &outbytesleft);
 	} while (rc == (size_t)-1 && errno == E2BIG);
 
 	if (len > MAX_BUF_SIZE) {
@@ -1570,7 +1571,8 @@ alloc_string:
 BYTE *
 Trspi_UNICODE_To_Native(BYTE *string, unsigned *size)
 {
-	char *ret, *ptr, *outbuf, tmpbuf[MAX_BUF_SIZE] = { 0, };
+	char *ret, *outbuf, tmpbuf[MAX_BUF_SIZE] = { 0, };
+	BSD_CONST char *ptr;
 	unsigned len = 0, tmplen;
 	iconv_t cd;
 	size_t rc, outbytesleft, inbytesleft;
@@ -1599,7 +1601,7 @@ Trspi_UNICODE_To_Native(BYTE *string, unsigned *size)
 		ptr = (char *)string;
 		errno = 0;
 
-		rc = iconv(cd, &ptr, &inbytesleft, &outbuf, &outbytesleft);
+		rc = iconv(cd, (BSD_CONST char **)&ptr, &inbytesleft, &outbuf, &outbytesleft);
 	} while (rc == (size_t)-1 && errno == E2BIG);
 
 	/* add terminating bytes of the correct width */
