@@ -11,6 +11,8 @@
 
 #include <stdio.h>
 #include <string.h>
+
+#include "trousers/tss.h"
 #include "tsplog.h"
 
 #ifdef TSS_DEBUG
@@ -48,4 +50,13 @@ LogBlobData(char *szDescriptor, unsigned long sizeOfBlob, unsigned char *blob)
 	fprintf(stdout, "%s\n", temp);
 }
 
+TSS_RESULT
+LogTSPERR(TSS_RESULT result, char *file, int line)
+{
+	if (getenv("TSS_DEBUG_OFF") == NULL)
+		fprintf(stderr, "%s %s %s:%d: 0x%x\n", "LOG_RETERR", APPID, file, line, result);
+
+	return (result | TSS_LAYER_TSP);
+}
+
 #endif
