@@ -32,7 +32,9 @@ PCRS_LIST_DECLARE;
 POLICY_LIST_DECLARE;
 RSAKEY_LIST_DECLARE;
 ENCDATA_LIST_DECLARE;
-DAA_LIST_DECLARE;
+DAACRED_LIST_DECLARE;
+DAAARAKEY_LIST_DECLARE;
+DAAISSUERKEY_LIST_DECLARE;
 NVSTORE_LIST_DECLARE;
 DELFAMILY_LIST_DECLARE;
 MIGDATA_LIST_DECLARE;
@@ -45,7 +47,7 @@ list_init(struct obj_list *list)
 }
 
 void
-obj_list_init()
+__tspi_obj_list_init()
 {
 	TPM_LIST_INIT();
 	CONTEXT_LIST_INIT();
@@ -54,7 +56,9 @@ obj_list_init()
 	POLICY_LIST_INIT();
 	RSAKEY_LIST_INIT();
 	ENCDATA_LIST_INIT();
-	DAA_LIST_INIT();
+	DAACRED_LIST_INIT();
+	DAAARAKEY_LIST_INIT();
+	DAAISSUERKEY_LIST_INIT();
 	NVSTORE_LIST_INIT();
 	DELFAMILY_LIST_INIT();
 	MIGDATA_LIST_INIT();
@@ -167,7 +171,6 @@ TSS_RESULT
 obj_list_remove(struct obj_list *list, void (*freeFcn)(void *), TSS_HOBJECT hObject, TSS_HCONTEXT tspContext)
 {
 	struct tsp_object *obj, *prev = NULL;
-	TSS_RESULT result = TSPERR(TSS_E_INVALID_HANDLE);
 
 	MUTEX_LOCK(list->lock);
 
@@ -184,14 +187,15 @@ obj_list_remove(struct obj_list *list, void (*freeFcn)(void *), TSS_HOBJECT hObj
 			else
 				list->head = obj->next;
 			free(obj);
-			result = TSS_SUCCESS;
-			break;
+
+			MUTEX_UNLOCK(list->lock);
+			return TSS_SUCCESS;
 		}
 	}
 
 	MUTEX_UNLOCK(list->lock);
 
-	return result;
+	return TSPERR(TSS_E_INVALID_HANDLE);
 }
 
 /* a generic routine for removing all members of a list who's tsp context
@@ -239,7 +243,9 @@ obj_close_context(TSS_HCONTEXT tspContext)
 	POLICY_LIST_CLOSE(tspContext);
 	RSAKEY_LIST_CLOSE(tspContext);
 	ENCDATA_LIST_CLOSE(tspContext);
-	DAA_LIST_CLOSE(tspContext);
+	DAACRED_LIST_CLOSE(tspContext);
+	DAAARAKEY_LIST_CLOSE(tspContext);
+	DAAISSUERKEY_LIST_CLOSE(tspContext);
 	NVSTORE_LIST_CLOSE(tspContext);
 	DELFAMILY_LIST_CLOSE(tspContext);
 	MIGDATA_LIST_CLOSE(tspContext);
