@@ -61,7 +61,7 @@ obj_policy_add(TSS_HCONTEXT tsp_context, UINT32 type, TSS_HOBJECT *phObject)
 }
 
 void
-policy_free(void *data)
+__tspi_policy_free(void *data)
 {
 	struct tr_policy_obj *policy = (struct tr_policy_obj *)data;
 
@@ -77,7 +77,7 @@ obj_policy_remove(TSS_HOBJECT hObject, TSS_HCONTEXT tspContext)
 {
 	TSS_RESULT result;
 
-	if ((result = obj_list_remove(&policy_list, &policy_free, hObject, tspContext)))
+	if ((result = obj_list_remove(&policy_list, &__tspi_policy_free, hObject, tspContext)))
 		return result;
 
 	obj_lists_remove_policy_refs(hObject, tspContext);
@@ -301,6 +301,25 @@ done:
 	return result;
 }
 
+TSS_RESULT
+obj_policy_is_secret_set(TSS_HPOLICY hPolicy, TSS_BOOL *secretSet) 
+{
+	struct tsp_object *obj;
+	struct tr_policy_obj *policy;
+	TSS_RESULT result = TSS_SUCCESS;
+
+	if ((obj = obj_list_get_obj(&policy_list, hPolicy)) == NULL)
+		return TSPERR(TSS_E_INVALID_HANDLE);
+
+	policy = (struct tr_policy_obj *)obj->data;
+	
+	*secretSet = policy->SecretSet;
+	obj_list_put(&policy_list);
+	
+	return result;
+}
+	
+
 TSS_RESULT
 obj_policy_set_secret(TSS_HPOLICY hPolicy, TSS_FLAG mode, UINT32 size, BYTE *data)
 {
@@ -882,7 +901,8 @@ obj_policy_get_xsap_params(TSS_HPOLICY hPolicy,
 			   TSS_CALLBACK *cb_xor,
 			   TSS_CALLBACK *cb_hmac,
 			   TSS_CALLBACK *cb_sealx,
-			   UINT32 *mode)
+			   UINT32 *mode,
+			   TSS_BOOL new_secret)
 {
 	struct tsp_object *obj;
 	struct tr_policy_obj *policy;
@@ -901,7 +921,7 @@ obj_policy_get_xsap_params(TSS_HPOLICY hPolicy,
 		result = TSPERR(TSS_E_INVALID_OBJ_ACCESS);
 		goto done;
 	}
-
+#ifdef TSS_BUILD_DELEGATION
 	/* if the delegation index or blob is set, check to see if the command is delegated, if so,
 	 * return the blob or index as the secret data */
 	if (command && (policy->delegationType != TSS_DELEGATIONTYPE_NONE)) {
@@ -931,7 +951,7 @@ obj_policy_get_xsap_params(TSS_HPOLICY hPolicy,
 			*et = TPM_ET_DEL_ROW;
 		}
 	}
-
+#endif
 	/* Either this is a policy set to mode callback, in which case both xor and hmac addresses
 	 * must be set, or this is an encrypted data object's policy, where its mode is independent
 	 * of whether a sealx callback is set */
@@ -949,12 +969,23 @@ obj_policy_get_xsap_params(TSS_HPOLICY hPolicy,
 		cb_hmac->callback = policy->Tspicb_CallbackHMACAuth;
 		cb_hmac->appData = policy->hmacAppData;
 		cb_hmac->alg = policy->hmacAlg;
+#ifdef TSS_BUILD_SEALX
 	} else if (cb_sealx && policy->Tspicb_CallbackSealxMask) {
 		cb_sealx->callback = policy->Tspicb_CallbackSealxMask;
 		cb_sealx->appData = policy->sealxAppData;
 		cb_sealx->alg = policy->sealxAlg;
+#endif
 	}
 
+	if ((policy->SecretMode == TSS_SECRET_MODE_POPUP) &&
+	    (policy->SecretSet == FALSE)) {
+		if ((result = popup_GetSecret(new_secret,
+					      policy->hashMode,
+					      policy->popupString,
+					      policy->Secret)))
+			goto done;
+			policy->SecretSet = TRUE;
+	}	
 	memcpy(secret, policy->Secret, TPM_SHA1_160_HASH_LEN);
 	*mode = policy->SecretMode;
 done:
