@@ -82,7 +82,7 @@ obj_rsakey_add(TSS_HCONTEXT tspContext, TSS_FLAG initFlags, TSS_HOBJECT *phObjec
 			break;
 	}
 
-	if (initFlags & TSS_KEY_EMPTY_KEY)
+	if (initFlags == TSS_KEY_EMPTY_KEY)
 		goto add_key;
 
 	memset(&rsaKeyParms, 0, sizeof(TCPA_RSA_KEY_PARMS));
@@ -754,6 +754,9 @@ obj_rsakey_get_ss(TSS_HKEY hKey, UINT32 *ss)
 		case TCPA_SS_RSASSAPKCS1v15_DER:
 			*ss = TSS_SS_RSASSAPKCS1V15_DER;
 			break;
+		case TCPA_SS_RSASSAPKCS1v15_INFO:
+			*ss = TSS_SS_RSASSAPKCS1V15_INFO;
+			break;
 		default:
 			*ss = rsakey->key.algorithmParms.sigScheme;
 			break;
@@ -793,6 +796,9 @@ obj_rsakey_set_ss(TSS_HKEY hKey, UINT32 ss)
 		case TSS_SS_RSASSAPKCS1V15_DER:
 			rsakey->key.algorithmParms.sigScheme = TCPA_SS_RSASSAPKCS1v15_DER;
 			break;
+		case TSS_SS_RSASSAPKCS1V15_INFO:
+			rsakey->key.algorithmParms.sigScheme = TCPA_SS_RSASSAPKCS1v15_INFO;
+			break;
 		default:
 			rsakey->key.algorithmParms.sigScheme = ss;
 			break;
@@ -1458,7 +1464,7 @@ obj_rsakey_get_tcs_handle(TSS_HKEY hKey, TCS_KEY_HANDLE *tcsHandle)
 
 	obj_list_put(&rsakey_list);
 
-	return TSS_SUCCESS;
+	return result;
 }
 
 TSS_RESULT
@@ -1718,7 +1724,7 @@ obj_rsakey_set_srk_pubkey(BYTE *pubkey)
 	struct tsp_object *obj, *prev = NULL;
 	struct obj_list *list = &rsakey_list;
 	struct tr_rsakey_obj *rsakey;
-	TSS_RESULT result = TSPERR(TSS_E_INVALID_HANDLE);
+	TSS_RESULT result;
 
 	MUTEX_LOCK(list->lock);
 
@@ -1728,13 +1734,14 @@ obj_rsakey_set_srk_pubkey(BYTE *pubkey)
 		/* we found the SRK, set this data as its public key */
 		if (rsakey->tcsHandle == TPM_KEYHND_SRK) {
 			result = rsakey_set_pubkey(rsakey, pubkey);
-			break;
+			MUTEX_UNLOCK(list->lock);
+			return result;
 		}
 	}
 
 	MUTEX_UNLOCK(list->lock);
 
-	return result;
+	return TSPERR(TSS_E_INVALID_HANDLE);
 }
 
 TSS_RESULT
@@ -1808,7 +1815,7 @@ done:
 }
 
 void
-rsakey_free(void *data)
+__tspi_rsakey_free(void *data)
 {
 	struct tr_rsakey_obj *rsakey = (struct tr_rsakey_obj *)data;
 
@@ -1826,7 +1833,7 @@ obj_rsakey_remove(TSS_HOBJECT hObject, TSS_HCONTEXT tspContext)
 {
 	TSS_RESULT result;
 
-	if ((result = obj_list_remove(&rsakey_list, &rsakey_free, hObject, tspContext)))
+	if ((result = obj_list_remove(&rsakey_list, &__tspi_rsakey_free, hObject, tspContext)))
 		return result;
 
 	return TSS_SUCCESS;
