@@ -19,6 +19,21 @@
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <assert.h>
+#include <fcntl.h>
+#include <limits.h>
+#include <netdb.h>
+#if defined (HAVE_BYTEORDER_H)
+#include <sys/byteorder.h>
+#elif defined(HAVE_ENDIAN_H)
+#include <endian.h>
+#define LE_16 htole16
+#define LE_32 htole32
+#define LE_64 htole64
+#else
+#define LE_16(x) (x)
+#define LE_32(x) (x)
+#define LE_64(x) (x)
+#endif
 
 #include "trousers/tss.h"
 #include "trousers/trousers.h"
@@ -30,6 +45,11 @@
 
 static int user_ps_fd = -1;
 static MUTEX_DECLARE_INIT(user_ps_lock);
+#if (defined (__FreeBSD__) || defined (__OpenBSD__))
+static MUTEX_DECLARE_INIT(user_ps_path);
+#endif
+static struct flock fl;
+
 
 /*
  * Determine the default path to the persistent storage file and create it if it doesn't exist.
@@ -37,9 +57,12 @@ static MUTEX_DECLARE_INIT(user_ps_lock);
 TSS_RESULT
 get_user_ps_path(char **file)
 {
-	TSS_RESULT result = TSPERR(TSS_E_INTERNAL_ERROR);
+	TSS_RESULT result;
 	char *file_name = NULL, *home_dir = NULL;
-	struct passwd pw, *pwp;
+	struct passwd *pwp;
+#if (defined (__linux) || defined (linux) || defined(__GLIBC__))
+	struct passwd pw;
+#endif
 	struct stat stat_buf;
 	char buf[PASSWD_BUFSIZE];
 	uid_t euid;
@@ -49,11 +72,25 @@ get_user_ps_path(char **file)
 		*file = strdup(file_name);
 		return (*file) ? TSS_SUCCESS : TSPERR(TSS_E_OUTOFMEMORY);
 	}
+#if (defined (__FreeBSD__) || defined (__OpenBSD__))
+	MUTEX_LOCK(user_ps_path);
+#endif
 
 	euid = geteuid();
 
+#if defined (SOLARIS)
+	/*
+         * Solaris keeps user PS in a local directory instead of
+         * in the user's home directory, which may be shared
+         * by multiple systems.
+         *
+         * The directory path on Solaris is /var/tpm/userps/[EUID]/
+         */
+        rc = snprintf(buf, sizeof (buf), "%s/%d", TSS_USER_PS_DIR, euid);
+#else
 	setpwent();
 	while (1) {
+#if (defined (__linux) || defined (linux) || defined(__GLIBC__))
 		rc = getpwent_r(&pw, buf, PASSWD_BUFSIZE, &pwp);
 		if (rc) {
 			LogDebugFn("USER PS: Error getting path to home directory: getpwent_r: %s",
@@ -62,20 +99,31 @@ get_user_ps_path(char **file)
 			return TSPERR(TSS_E_INTERNAL_ERROR);
 		}
 
-		if (euid == pwp->pw_uid) {
-			home_dir = strdup(pwp->pw_dir);
-			break;
+#elif (defined (__FreeBSD__) || defined (__OpenBSD__))
+		if ((pwp = getpwent()) == NULL) {
+			LogDebugFn("USER PS: Error getting path to home directory: getpwent: %s",
+                                   strerror(rc));
+			endpwent();
+			MUTEX_UNLOCK(user_ps_path);
+			return TSPERR(TSS_E_INTERNAL_ERROR);
 		}
-	}
-	endpwent();
+#endif
+		if (euid == pwp->pw_uid) {
+                        home_dir = strdup(pwp->pw_dir);
+                        break;
+                }
+        }
+        endpwent();
 
 	if (!home_dir)
 		return TSPERR(TSS_E_OUTOFMEMORY);
 
 	/* Tack on TSS_USER_PS_DIR and see if it exists */
-	rc = snprintf(buf, PASSWD_BUFSIZE, "%s/%s", home_dir, TSS_USER_PS_DIR);
-	if (rc == PASSWD_BUFSIZE) {
+	rc = snprintf(buf, sizeof (buf), "%s/%s", home_dir, TSS_USER_PS_DIR);
+#endif /* SOLARIS */
+	if (rc == sizeof (buf)) {
 		LogDebugFn("USER PS: Path to file too long! (> %d bytes)", PASSWD_BUFSIZE);
+		result = TSPERR(TSS_E_INTERNAL_ERROR);
 		goto done;
 	}
 
@@ -83,23 +131,30 @@ get_user_ps_path(char **file)
 	if ((rc = stat(buf, &stat_buf)) == -1) {
 		if (errno == ENOENT) {
 			errno = 0;
-			/* Create the base directory, $HOME/.trousers */
+			/* Create the user's ps directory if it is not there. */
 			if ((rc = mkdir(buf, 0700)) == -1) {
 				LogDebugFn("USER PS: Error creating dir: %s: %s", buf,
 					   strerror(errno));
+				result = TSPERR(TSS_E_INTERNAL_ERROR);
 				goto done;
 			}
 		} else {
 			LogDebugFn("USER PS: Error stating dir: %s: %s", buf, strerror(errno));
+			result = TSPERR(TSS_E_INTERNAL_ERROR);
 			goto done;
 		}
 	}
 
 	/* Directory exists or has been created, return the path to the file */
-	rc = snprintf(buf, PASSWD_BUFSIZE, "%s/%s/%s", home_dir, TSS_USER_PS_DIR,
+#if defined (SOLARIS)
+	rc = snprintf(buf, sizeof (buf), "%s/%d/%s", TSS_USER_PS_DIR, euid,
 		      TSS_USER_PS_FILE);
-	if (rc == PASSWD_BUFSIZE) {
-		LogDebugFn("USER PS: Path to file too long! (> %d bytes)", PASSWD_BUFSIZE);
+#else
+	rc = snprintf(buf, sizeof (buf), "%s/%s/%s", home_dir, TSS_USER_PS_DIR,
+		      TSS_USER_PS_FILE);
+#endif
+	if (rc == sizeof (buf)) {
+		LogDebugFn("USER PS: Path to file too long! (> %zd bytes)", sizeof (buf));
 	} else
 		*file = strdup(buf);
 
@@ -120,12 +175,12 @@ get_file(int *fd)
 
 	/* check the global file handle first.  If it exists, lock it and return */
 	if (user_ps_fd != -1) {
-		if ((rc = flock(user_ps_fd, LOCK_EX))) {
+		fl.l_type = F_WRLCK;
+		if ((rc = fcntl(user_ps_fd, F_SETLKW, &fl))) {
 			LogDebug("USER PS: failed to lock file: %s", strerror(errno));
 			MUTEX_UNLOCK(user_ps_lock);
 			return TSPERR(TSS_E_INTERNAL_ERROR);
 		}
-
 		*fd = user_ps_fd;
 		return TSS_SUCCESS;
 	}
@@ -144,8 +199,8 @@ get_file(int *fd)
 		MUTEX_UNLOCK(user_ps_lock);
 		return TSPERR(TSS_E_INTERNAL_ERROR);
 	}
-
-	if ((rc = flock(user_ps_fd, LOCK_EX))) {
+	fl.l_type = F_WRLCK;
+	if ((rc = fcntl(user_ps_fd, F_SETLKW, &fl))) {
 		LogDebug("USER PS: failed to get lock of %s: %s", file_name, strerror(errno));
 		free(file_name);
 		close(user_ps_fd);
@@ -167,7 +222,8 @@ put_file(int fd)
 	fsync(fd);
 
 	/* release the file lock */
-	if ((rc = flock(fd, LOCK_UN))) {
+	fl.l_type = F_UNLCK;
+	if ((rc = fcntl(fd, F_SETLKW, &fl))) {
 		LogDebug("USER PS: failed to unlock file: %s", strerror(errno));
 		rc = -1;
 	}
@@ -192,7 +248,7 @@ psfile_is_key_registered(int fd, TSS_UUID *uuid, TSS_BOOL *answer)
 
 	if ((result = psfile_get_cache_entry_by_uuid(fd, uuid, &tmp)) == TSS_SUCCESS)
 		*answer = TRUE;
-	else if (result == TSPERR(TSS_E_PS_KEY_NOTFOUND))
+	else if (result == (TSS_E_PS_KEY_NOTFOUND | TSS_LAYER_TSP))
 		*answer = FALSE;
         else
                 return result;
@@ -255,6 +311,11 @@ psfile_get_key_by_uuid(int fd, TSS_UUID *uuid, BYTE *key)
 		return TSPERR(TSS_E_INTERNAL_ERROR);
 	}
 
+	if (tmp.blob_size > 4096) {
+		LogError("Blob size greater than 4096! Size:  %d",
+			  tmp.blob_size);
+		return TSPERR(TSS_E_INTERNAL_ERROR);
+	}
 	if ((rc = read_data(fd, buf, tmp.blob_size))) {
 		LogDebugFn("Blob read from disk failed.");
 		return rc;
@@ -288,6 +349,12 @@ psfile_get_key_by_pub(int fd, TSS_UUID *uuid, UINT32 pub_size, BYTE *pub, BYTE *
 		return TSPERR(TSS_E_INTERNAL_ERROR);
 	}
 
+	if (tmp.blob_size > 4096) {
+		LogError("Blob size greater than 4096! Size:  %d",
+			  tmp.blob_size);
+		return TSPERR(TSS_E_INTERNAL_ERROR);
+	}
+
 	if ((result = read_data(fd, buf, tmp.blob_size))) {
 		LogDebugFn("Blob read from disk failed.");
 		return result;
@@ -331,6 +398,7 @@ psfile_change_num_keys(int fd, BYTE increment)
 		LogDebug("read of %zd bytes: %s", sizeof(UINT32), strerror(errno));
 		return TSPERR(TSS_E_INTERNAL_ERROR);
 	}
+	num_keys = LE_32(num_keys);
 
 	if (increment)
 		num_keys++;
@@ -343,6 +411,7 @@ psfile_change_num_keys(int fd, BYTE increment)
 		return TSPERR(TSS_E_INTERNAL_ERROR);
 	}
 
+	num_keys = LE_32(num_keys);
 	if ((result = write_data(fd, (void *)&num_keys, sizeof(UINT32)))) {
 		LogDebug("%s", __FUNCTION__);
 		return result;
@@ -464,16 +533,20 @@ psfile_write_key(int fd,
 	}
 
 	/* [UINT16   pub_data_size0  ] yes */
+	pub_key_size = LE_16(pub_key_size);
         if ((result = write_data(fd, &pub_key_size, sizeof(UINT16)))) {
 		LogDebug("%s", __FUNCTION__);
 		goto done;
 	}
+	pub_key_size = LE_16(pub_key_size);
 
 	/* [UINT16   blob_size0      ] yes */
+	key_blob_size = LE_16(key_blob_size);
         if ((result = write_data(fd, &key_blob_size, sizeof(UINT16)))) {
 		LogDebug("%s", __FUNCTION__);
 		goto done;
 	}
+	key_blob_size = LE_16(key_blob_size);
 
 	/* [UINT32   vendor_data_size0 ] yes */
         if ((result = write_data(fd, &zero, sizeof(UINT32)))) {
@@ -482,10 +555,12 @@ psfile_write_key(int fd,
 	}
 
 	/* [UINT16   cache_flags0    ] yes */
+	cache_flags = LE_16(cache_flags);
         if ((result = write_data(fd, &cache_flags, sizeof(UINT16)))) {
 		LogDebug("%s", __FUNCTION__);
 		goto done;
 	}
+	cache_flags = LE_16(cache_flags);
 
 	/* [BYTE[]   pub_data0       ] no */
         if ((result = write_data(fd, (void *)key.pubKey.key, pub_key_size))) {
@@ -651,6 +726,7 @@ psfile_get_all_cache_entries(int fd, UINT32 *size, struct key_disk_cache **c)
 			LogDebug("%s", __FUNCTION__);
 			goto err_exit;
 		}
+		tmp[i].pub_data_size = LE_16(tmp[i].pub_data_size);
 
 		DBG_ASSERT(tmp[i].pub_data_size <= 2048);
 
@@ -659,6 +735,7 @@ psfile_get_all_cache_entries(int fd, UINT32 *size, struct key_disk_cache **c)
 			LogDebug("%s", __FUNCTION__);
 			goto err_exit;
 		}
+		tmp[i].blob_size = LE_16(tmp[i].blob_size);
 
 		DBG_ASSERT(tmp[i].blob_size <= 4096);
 
@@ -667,12 +744,14 @@ psfile_get_all_cache_entries(int fd, UINT32 *size, struct key_disk_cache **c)
 			LogDebug("%s", __FUNCTION__);
 			goto err_exit;
 		}
+		tmp[i].vendor_data_size = LE_32(tmp[i].vendor_data_size);
 
 		/* cache flags */
 		if ((result = read_data(fd, &tmp[i].flags, sizeof(UINT16)))) {
 			LogDebug("%s", __FUNCTION__);
 			goto err_exit;
 		}
+		tmp[i].flags = LE_16(tmp[i].flags);
 
 		/* fast forward over the pub key */
 		offset = lseek(fd, tmp[i].pub_data_size, SEEK_CUR);
@@ -997,6 +1076,8 @@ psfile_get_num_keys(int fd)
 		num_keys = 0;
 	}
 
+	/* The system PS file is written in little-endian */
+	num_keys = LE_32(num_keys);
 	return num_keys;
 }
 
@@ -1075,7 +1156,7 @@ psfile_get_cache_entry_by_uuid(int fd, TSS_UUID *uuid, struct key_disk_cache *c)
 			LogDebug("%s", __FUNCTION__);
 			return result;
 		}
-
+		c->pub_data_size = LE_16(c->pub_data_size);
 		DBG_ASSERT(c->pub_data_size <= 2048 && c->pub_data_size > 0);
 
 		/* blob size */
@@ -1083,7 +1164,7 @@ psfile_get_cache_entry_by_uuid(int fd, TSS_UUID *uuid, struct key_disk_cache *c)
 			LogDebug("%s", __FUNCTION__);
 			return result;
 		}
-
+		c->blob_size = LE_16(c->blob_size); 
 		DBG_ASSERT(c->blob_size <= 4096 && c->blob_size > 0);
 
 		/* vendor data size */
@@ -1091,12 +1172,14 @@ psfile_get_cache_entry_by_uuid(int fd, TSS_UUID *uuid, struct key_disk_cache *c)
 			LogDebug("%s", __FUNCTION__);
 			return result;
 		}
+		c->vendor_data_size = LE_32(c->vendor_data_size); 
 
 		/* cache flags */
 		if ((result = read_data(fd, &c->flags, sizeof(UINT16)))) {
 			LogDebug("%s", __FUNCTION__);
 			return result;
 		}
+		c->flags = LE_16(c->flags); 
 
 		/* fast forward over the pub key */
 		offset = lseek(fd, c->pub_data_size, SEEK_CUR);
@@ -1164,6 +1247,7 @@ psfile_get_cache_entry_by_pub(int fd, UINT32 pub_size, BYTE *pub, struct key_dis
 			return result;
 		}
 
+		c->pub_data_size = LE_16(c->pub_data_size);
 		DBG_ASSERT(c->pub_data_size <= 2048 && c->pub_data_size > 0);
 
 		/* blob size */
@@ -1172,6 +1256,7 @@ psfile_get_cache_entry_by_pub(int fd, UINT32 pub_size, BYTE *pub, struct key_dis
 			return result;
 		}
 
+		c->blob_size = LE_16(c->blob_size);
 		DBG_ASSERT(c->blob_size <= 4096 && c->blob_size > 0);
 
 		/* vendor data size */
@@ -1179,12 +1264,14 @@ psfile_get_cache_entry_by_pub(int fd, UINT32 pub_size, BYTE *pub, struct key_dis
 			LogDebug("%s", __FUNCTION__);
 			return result;
 		}
+		c->vendor_data_size = LE_32(c->vendor_data_size);
 
 		/* cache flags */
 		if ((result = read_data(fd, &c->flags, sizeof(UINT16)))) {
 			LogDebug("%s", __FUNCTION__);
 			return result;
 		}
+		c->flags = LE_16(c->flags);
 
 		if (c->pub_data_size == pub_size) {
 			/* read in the pub key */
