@@ -110,7 +110,7 @@ create_owner_delegation(TSS_HTPM       hTpm,
 	Trspi_HashCtx hashCtx;
 	TCPA_DIGEST digest;
 	UINT32 blobSize;
-	BYTE *blob;
+	BYTE *blob = NULL;
 	TSS_RESULT result;
 	struct authsess *xsap = NULL;
 
@@ -129,7 +129,7 @@ create_owner_delegation(TSS_HTPM       hTpm,
 	if (type != TSS_DELEGATIONTYPE_OWNER)
 		return TSPERR(TSS_E_BAD_PARAMETER);
 
-	if ((result = build_delegate_public_info(bLabel, hPcrs, hFamily, hDelegation,
+	if ((result = __tspi_build_delegate_public_info(bLabel, hPcrs, hFamily, hDelegation,
 			&publicInfoSize, &publicInfo)))
 		return result;
 
@@ -176,6 +176,7 @@ create_owner_delegation(TSS_HTPM       hTpm,
 done:
 	authsess_free(xsap);
 	free(publicInfo);
+	free(blob);
 
 	return result;
 }
@@ -196,7 +197,7 @@ create_key_delegation(TSS_HKEY       hKey,
 	Trspi_HashCtx hashCtx;
 	TCPA_DIGEST digest;
 	UINT32 blobSize;
-	BYTE *blob;
+	BYTE *blob = NULL;
 	TSS_RESULT result;
 	struct authsess *xsap = NULL;
 
@@ -215,7 +216,7 @@ create_key_delegation(TSS_HKEY       hKey,
 	if ((result = obj_rsakey_get_tcs_handle(hKey, &tcsKeyHandle)))
 		return result;
 
-	if ((result = build_delegate_public_info(bLabel, hPcrs, hFamily, hDelegation,
+	if ((result = __tspi_build_delegate_public_info(bLabel, hPcrs, hFamily, hDelegation,
 			&publicInfoSize, &publicInfo)))
 		return result;
 
@@ -259,6 +260,7 @@ create_key_delegation(TSS_HKEY       hKey,
 						blob);
 
 done:
+	free(blob);
 	authsess_free(xsap);
 	free(publicInfo);
 
@@ -362,7 +364,7 @@ done:
 }
 
 TSS_RESULT
-build_delegate_public_info(BYTE           bLabel,
+__tspi_build_delegate_public_info(BYTE           bLabel,
 			   TSS_HPCRS      hPcrs,
 			   TSS_HDELFAMILY hFamily,
 			   TSS_HPOLICY    hDelegation,
@@ -437,7 +439,7 @@ Transport_Delegate_Manage(TSS_HCONTEXT tspContext,              /* in */
 	TSS_RESULT result;
 	UINT32 handlesLen = 0, decLen, dataLen;
 	UINT64 offset;
-	BYTE *data, *dec;
+	BYTE *data, *dec = NULL;
 
 
 	if ((result = obj_context_transport_init(tspContext)))
@@ -500,8 +502,7 @@ Transport_Delegate_CreateKeyDelegation(TSS_HCONTEXT tspContext,         /* in */
 	TPM_DIGEST pubKeyHash;
 	Trspi_HashCtx hashCtx;
 	UINT64 offset;
-	BYTE *data, *dec;
-
+	BYTE *data, *dec = NULL;
 
 	if ((result = obj_context_transport_init(tspContext)))
 		return result;
@@ -568,8 +569,7 @@ Transport_Delegate_CreateOwnerDelegation(TSS_HCONTEXT tspContext,       /* in */
 	TSS_RESULT result;
 	UINT32 handlesLen = 0, decLen, dataLen;
 	UINT64 offset;
-	BYTE *data, *dec;
-
+	BYTE *data, *dec = NULL;
 
 	if ((result = obj_context_transport_init(tspContext)))
 		return result;
@@ -620,10 +620,9 @@ Transport_Delegate_LoadOwnerDelegation(TSS_HCONTEXT tspContext, /* in */
 				       TPM_AUTH *ownerAuth)           /* in, out */
 {
 	TSS_RESULT result;
-	UINT32 handlesLen = 0, dataLen;
+	UINT32 handlesLen = 0, dataLen, decLen;
 	UINT64 offset;
-	BYTE *data;
-
+	BYTE *data, *dec = NULL;
 
 	if ((result = obj_context_transport_init(tspContext)))
 		return result;
@@ -644,11 +643,12 @@ Transport_Delegate_LoadOwnerDelegation(TSS_HCONTEXT tspContext, /* in */
 	if ((result = obj_context_transport_execute(tspContext,
 						    TPM_ORD_Delegate_LoadOwnerDelegation, dataLen,
 						    data, NULL, &handlesLen, NULL, ownerAuth,
-						    NULL, NULL, NULL))) {
+						    NULL, &decLen, &dec))) {
 		free(data);
 		return result;
 	}
 	free(data);
+	free(dec);
 
 	return result;
 }
@@ -663,8 +663,7 @@ Transport_Delegate_ReadTable(TSS_HCONTEXT tspContext,           /* in */
 	TSS_RESULT result;
 	UINT32 handlesLen = 0, decLen;
 	UINT64 offset;
-	BYTE *dec;
-
+	BYTE *dec = NULL;
 
 	if ((result = obj_context_transport_init(tspContext)))
 		return result;
@@ -716,7 +715,7 @@ Transport_Delegate_UpdateVerificationCount(TSS_HCONTEXT tspContext,     /* in */
 	TSS_RESULT result;
 	UINT32 handlesLen = 0, decLen, dataLen;
 	UINT64 offset;
-	BYTE *data, *dec;
+	BYTE *data, *dec = NULL;
 
 
 	if ((result = obj_context_transport_init(tspContext)))
@@ -742,7 +741,6 @@ Transport_Delegate_UpdateVerificationCount(TSS_HCONTEXT tspContext,     /* in */
 	}
 	free(data);
 
-
 	offset = 0;
 	Trspi_UnloadBlob_UINT32(&offset, outputSize, dec);
 
@@ -765,9 +763,9 @@ Transport_Delegate_VerifyDelegation(TSS_HCONTEXT tspContext,    /* in */
 				    BYTE *delegate)           /* in */
 {
 	TSS_RESULT result;
-	UINT32 handlesLen = 0, dataLen;
+	UINT32 handlesLen = 0, dataLen, decLen;
 	UINT64 offset;
-	BYTE *data;
+	BYTE *data, *dec = NULL;
 
 
 	if ((result = obj_context_transport_init(tspContext)))
@@ -787,8 +785,9 @@ Transport_Delegate_VerifyDelegation(TSS_HCONTEXT tspContext,    /* in */
 
 	result = obj_context_transport_execute(tspContext, TPM_ORD_Delegate_VerifyDelegation,
 					       dataLen, data, NULL, &handlesLen, NULL, NULL, NULL,
-					       NULL, NULL);
+					       &decLen, &dec);
 	free(data);
+	free(dec);
 
 	return result;
 }
@@ -810,8 +809,7 @@ Transport_DSAP(TSS_HCONTEXT tspContext,		/* in */
 	TPM_DIGEST pubKeyHash;
 	Trspi_HashCtx hashCtx;
 	UINT64 offset;
-	BYTE *data, *dec;
-
+	BYTE *data, *dec = NULL;
 
 	if ((result = obj_context_transport_init(tspContext)))
 		return result;
@@ -860,6 +858,7 @@ Transport_DSAP(TSS_HCONTEXT tspContext,		/* in */
 	Trspi_UnloadBlob(&offset, sizeof(TPM_NONCE), dec, nonceEven->nonce);
 	Trspi_UnloadBlob(&offset, sizeof(TPM_NONCE), dec, nonceEvenDSAP->nonce);
 
+	free(dec);
 
 	return result;
 }
